EVENT_TEMPLATE = {
    "anonymousId": "",
    "messageId": "",
    "receivedAt": "",
    "sentAt": "",
    "timestamp": "",
    "type": "",
    "userId": "",
    "version": 1,
}

API_PATHS = {
    "process_event": "/events/process/",
}
